/* The Cantus project.
 * (c)2002, 2003 by Samuel Abels (spam debain org)
 * This project's homepage is: http://www.debain.org/cantus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <libintl.h>
#include "engine_conditions.h"
#include "engine_ruleset.h"

#define _(String) gettext (String)
#define gettext_noop(String) (String)
#define N_(String) gettext_noop (String)


/***************************************************************************************
 * STATIC CONDITIONS
 ***************************************************************************************/
static short int cond_always()
{
  return(TRUE);
}


static short int cond_shellpattern_match(const char *str, const char *pattern)
{
  return(shellpattern_match(str, pattern, TRUE));
}


static short int cond_regexp_match(const char *str, const char *regexp)
{
  if (regexp_match(str, regexp, TRUE) == 0)
    return(TRUE);
  return(FALSE);
}
/***************************************************************************************
 * END STATIC CONDITIONS
 ***************************************************************************************/


static void *condfunc_data_get(int row, int col)
{
  typedef struct {
    int       type;
    char      *description;
    CondFunc  func;
  } CondFuncStruct;
  CondFuncStruct condlist[] = {
    { COND_ALWAYS,              N_("Always:"),                        (CondFunc)cond_always },
    { COND_SHELLPATTERN_MATCH,  N_("%s matches the pattern \"%s\":"), (CondFunc)cond_shellpattern_match },
    { COND_REGEXP_MATCH,        N_("%s matches the RegExp \"%s\":"),  (CondFunc)cond_regexp_match },
    
    { -1, NULL, NULL }
  };
  switch (col) {
  case 0:
    return((void*)condlist[row].type);
  
  case 1:
    return(_(condlist[row].description));
  
  case 2:
    return(condlist[row].func);
  }
}


void condition_describe(Condition *cond, void *arg1)
{
  int i = -1;
  
  g_assert(cond != NULL);
  
  while (condfunc_data_get(++i, 1)) {
    if ((int)condfunc_data_get(i, 0) != cond->type)
      continue;
    snprintf(cond->descr, 1023, (char*)condfunc_data_get(i, 1), field_name_get(cond->field), (char*)arg1);
    break;
  }
}


short int cond_is_true(int type, FileInfo *info, int field, void *data1)
{
  int tag = (field & (T_ID3V1 | T_ID3V2 | T_OGG)) >> 11;
  typedef struct {
    int       field;
    char      *dest;
  } Pair;
  Pair map[] = {
    { F_FILENAME, strrchr(info->filename, '/') ? strrchr(info->filename, '/') + 1 : info->filename },
    { F_ARTIST,   info->tag[tag]->artist },
    { F_SONG,     info->tag[tag]->title },
    { F_ALBUM,    info->tag[tag]->album },
    { F_TRACK,    info->tag[tag]->track },
    { F_YEAR,     info->tag[tag]->year },
    { F_GENRE,    info->tag[tag]->genre },
    { F_COMMENT,  info->tag[tag]->comment },
    { 0, NULL }
  };
  int i = -1;
  CondFunc func;
  
  if (!info)
    return(FALSE);
  
  /* Walk through the function list, looking for the matching function. */
  while (condfunc_data_get(++i, 1)) {
    if ((int)condfunc_data_get(i, 0) == -1) {
      fprintf(stderr, "Error: condition_is_true(): Invalid condition identifier found!\n");
      return;
    }
    if ((int)condfunc_data_get(i, 0) != type)
      continue;
    func = (CondFunc)condfunc_data_get(i, 2);
  }
  
  /* Walk through the field list, executing the condition on the matching field. */
  i = -1;
  while (map[++i].dest) {
    if ((field & map[i].field) != map[i].field)
      continue;
    return(func(map[i].dest, data1));
  }
  
  /* Ending up here, we have nothing to match, so we return TRUE. */
  return(TRUE);
}
