/* The Cantus project.
 * (c)2002, 2003 by Samuel Abels (spam debain org)
 * This project's homepage is: http://www.debain.org/cantus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <pthread.h>
#include "engine_init.h"
#include "engine_jobs.h"

extern Engine engine;
extern Cfg cfg;


/***************************************************************************************
 * STATICS
 ***************************************************************************************/
static void *engine_loop(void *trash)
{
  pthread_mutex_lock(&engine.mut);
  while (!engine.shutdown) {
    pthread_cond_wait(&engine.cond, &engine.mut);
    job_proceed_all();
  }
  pthread_mutex_unlock(&engine.mut);
  pthread_exit(0);
}
/***************************************************************************************
 * END STATICS
 ***************************************************************************************/


void engine_init(void)
{
  /* Supported filetypes. */
  cfg.file_pattern[0]     = strdup("*.mp3");
  cfg.file_pattern[1]     = strdup("*.flac");
#ifdef ENABLE_OGG
  cfg.file_pattern[2]     = strdup("*.ogg");
#endif
  cfg.file_pattern[3]     = NULL;
  
  /* Default configuration. */
  cfg.defaulttag_v2       = TRUE;
  cfg.compat              = 4;
  cfg.use_unicode         = FALSE;
  cfg.preview_tageditor   = FALSE;
  cfg.preview_file2tag    = TRUE;
  cfg.preview_masseditor  = TRUE;
  
  /* Initialize the engine. */
  engine.shutdown         = FALSE;
  engine.queue            = NULL;
  engine.jobs             = NULL;
  engine.previewlist      = NULL;
  engine.progress         = -1;
  
  if (!rulesetlist_load(&engine.rulesetlist))
    fprintf(stderr, "Warning: engine_init(): Failed to load the ruleset list.\n");
  
  /* This thread will keep on and rename files and do all actions that the */
  /* GUI is asking it to do. */
  pthread_cond_init(&engine.cond, NULL);
  pthread_create(&engine.tid, NULL, engine_loop, (void *)NULL);
}


void engine_destroy(void)
{
  /* Save all rulesets to a configfile. */
  if (cfg.save_rulesets_on_exit)
    rulesetlist_save(engine.rulesetlist);
  int i = -1;
  while (cfg.file_pattern[++i])
    free((char *)cfg.file_pattern[i]);
  
  printf("Goodbye.\n");
}


void engine_stop(void)
{
  pthread_mutex_lock(&engine.mut);
  engine.shutdown = TRUE;
  pthread_cond_signal(&engine.cond);
  pthread_mutex_unlock(&engine.mut);
  printf("Engine shutdown requested... ");
  fflush(stdout);
}


void engine_wakeup(void)
{
  /* Signalize that something has to be done now. */
  pthread_mutex_lock(&engine.mut);
  pthread_cond_signal(&engine.cond);
  pthread_mutex_unlock(&engine.mut);
}
