/* The Cantus project.
 * (c)2002, 2003 by Samuel Abels (spam debain org)
 * This project's homepage is: http://www.debain.org/cantus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <glib.h>
#include <pthread.h>
#include <libintl.h>
#include "engine_fileinfo.h"
#include "engine_queue.h"
#include "engine_init.h"
#include "engine_fileinfo.h"

#define _(String) gettext (String)
#define gettext_noop(String) (String)
#define N_(String) gettext_noop (String)

extern Engine engine;

/***************************************************************************************
 * END STATICS
 ***************************************************************************************/
GHashTable *queue_hash_get(void)
{
  if (!engine.queue)
    engine.queue = g_hash_table_new(g_direct_hash, g_direct_equal);
  return(engine.queue);
}
/***************************************************************************************
 * END STATICS
 ***************************************************************************************/


/*
 * Append a file to the queue.
 */
void queue_append(const char *filename)
{
  FileInfo *info = NULL;
  
  if (!(info = fileinfo_get(filename, TRUE))) {
    fprintf(stderr, "Error: queue_append(): Failed getting the file info. (%s)\n", filename);
    return;
  }
  
  pthread_mutex_lock(&engine.mut);
  g_hash_table_insert(queue_hash_get(), info, info);
  pthread_mutex_unlock(&engine.mut);
}


/*
 * Remove the file from the queue.
 */
void queue_remove(FileInfo *info, void *trash)
{
  pthread_mutex_lock(&engine.mut);
  if (!g_hash_table_remove(queue_hash_get(), info)) {
    pthread_mutex_unlock(&engine.mut);
    fprintf(stderr, "Error: queue_remove(): Failed removing the file info.\n");
    return;
  }
  pthread_mutex_unlock(&engine.mut);
}


/*
 * Clear the queue.
 */
void queue_clear()
{
  GHashTable *queue = NULL;
  pthread_mutex_lock(&engine.mut);
  queue = engine.queue;
  engine.queue = NULL;
  pthread_mutex_unlock(&engine.mut);
  if (queue)
    g_hash_table_destroy(queue);
}
