/* The Cantus project.
 * (c)2002, 2003 by Samuel Abels (spam debain org)
 * This project's homepage is: http://www.debain.org/cantus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifndef RULES_H
#define RULES_H
#include "engine_fileinfo.h"

/* When adding new rules here, don't change the order */
/* but append the rule to the list end. Otherwise the */
/* ruleset files will become incompatible!            */
enum RULES {
  /* Flags. */
  RULE_STRIP_MULTIPLE_SPACES    = 1,
  RULE_REMOVE_LEADING_SPACES    = 2,
  RULE_REMOVE_TRAILING_SPACES   = 3,
  RULE_REMOVE_BEFORE_NUMBER     = 4,
  
  /* Capitol Letters. */
  RULE_UPPERCASE                = 5,
  RULE_UPPERCASE_WORD           = 6,
  RULE_UPPERCASE_FIRST_WORD     = 7,
  RULE_LOWERCASE                = 8,
  
  /* Static. */
  RULE_SET                      = 9,
  
  /* Modifiers. */
  RULE_REPLACE                  = 10,
  RULE_REPLACE_CI               = 11,
  
  /* Buffer Operations. */
  RULE_BUFFER_GRAB              = 12,
  RULE_BUFFER_COPY_TO           = 13,
  
  RULE_FORMAT_GRAB              = 14,
  
  NUM_RULES
};

enum TAGS {
  T_ID3V1 = 1 << 10,
  T_ID3V2 = 1 << 11,
  T_OGG   = 1 << 11
};

typedef void (*RuleFunc)(char *field, void *data1, void *data2);

typedef struct {
  int type;
  char descr[1024];
  short int data1type;
  void *data1;
  short int data2type;
  void *data2;
} Rule;
#endif

void rule_describe(Rule *rule, void *arg1, void *arg2);
void rule_apply(int type, FileInfo *info, int field, void *data1, void *data2);

RuleFunc rule_format_to_destination(char *src, const char *format, FileInfo *info);
