/* The Cantus project.
 * (c)2002, 2003 by Samuel Abels (spam debain org)
 * This project's homepage is: http://www.debain.org/cantus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifndef RULESET_H
#define RULESET_H

#include "engine_conditions.h"
#include "engine_rules.h"

#define RULESETLISTFILE ".cantusrules"
#define INT16 1
#define STR   2

enum FIELDS {
  F_FILENAME  =  1,
  F_ARTIST    =  1 << 1,
  F_SONG      =  1 << 2,
  F_ALBUM     =  1 << 3,
  F_TRACK     =  1 << 4,
  F_YEAR      =  1 << 5,
  F_GENRE     =  1 << 6,
  F_COMMENT   =  1 << 7,
  
  NUM_FIELDS
};

#define FIELDMASK (F_FILENAME | F_ARTIST | F_SONG | F_ALBUM | F_TRACK | F_YEAR | F_GENRE | F_COMMENT)

typedef struct {
  Condition           cond;
  Rule                rule;
  unsigned short int  dest;
  char                dest_descr[1024];
} RuleStatement;

typedef struct {
  char  name[1024];         /* A descriptive name to be shown in the GUI. */
  GList *statements;        /* The list of rules. */
  char  buffer[1024];       /* This is like a "clipboard" that can be used be rules. */
  int   ref;                /* A refcounter. */
} Ruleset;

Ruleset *ruleset_new(const char *name);
void ruleset_ref(Ruleset *ruleset);
void ruleset_unref(Ruleset *ruleset);
void ruleset_clear(Ruleset *ruleset);
Ruleset *ruleset_copy(Ruleset *ruleset);
Ruleset *ruleset_find(GList *rulelists, const char *name);

GFunc statement_proceed(RuleStatement *statement, void *data);
void statement_append(Ruleset *ruleset, int dest, int condition, int cond_field, short int conddata1type, void *conddata1,
                      int rule, short int ruledata1type, void *ruledata1, short int ruledata2type, void *ruledata2);
void statement_remove(Ruleset *ruleset, RuleStatement *statement);
void statement_destroy(RuleStatement *statement);

short int rulesetlist_save(GList *rulesetlist);
short int rulesetlist_load(GList **rulesetlist);

const char *field_name_get(int type);

#endif
