/* The Cantus project.
 * (c)2002, 2003 by Samuel Abels (spam debain org)
 * This project's homepage is: http://www.debain.org/cantus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <libintl.h>
#include <stdio.h>
#include "gui_init.h"
#include "gui_file2tag.h"
#include "engine_init.h"
#include "engine_jobs.h"

#define _(String) gettext (String)
#define gettext_noop(String) (String)
#define N_(String) gettext_noop (String)

extern GUI    gui;
extern Cfg    cfg;


typedef struct {
  char      *name;
  char      *dest;
} Pair;

/******************************************************************************
 * STATICS
 ******************************************************************************/
static PreviewDestroyFunc on_preview_destroy(void)
{
  gui.preview = NULL;
}


static gboolean field_get(int i, Tag *tag, Pair *pair)
{
  Pair map[] = {
    { "label_file2tag_artist",  tag->artist },
    { "label_file2tag_song",    tag->title },
    { "label_file2tag_album",   tag->album },
    { "label_file2tag_track",   tag->track },
    { "label_file2tag_year",    tag->year },
    { "label_file2tag_genre",   tag->genre },
    { "label_file2tag_comment", tag->comment },
    { 0, NULL }
  };
  
  if (!map[i].name)
    return(FALSE);
  
  pair->name = map[i].name;
  pair->dest = map[i].dest;
  
  return(TRUE);
}


static void gui_file2tag_history_push(const gchar *newtext)
{
  GtkCombo  *combo = GTK_COMBO(glade_xml_get_widget(gui.mainwindow_xml, "combo_file2tag_format"));
  GList     *item     = NULL;
  GList     *nextitem = NULL;
  gchar     *dead     = NULL;
  
  g_assert(combo != NULL);
  
  /* Walk through the history and remove the text, if it already exists. */
  item = gui.file2tag.history;
  while (item) {
    nextitem = item->next;
    if (strcmp(newtext, (gchar*)item->data) == 0) {
      dead = (gchar*)item->data;
      gui.file2tag.history = g_list_remove_link(gui.file2tag.history, item);
      g_list_free_1(item);
      free(dead);
    }
    item = nextitem;
  }
  
  /* Put the new pattern into a doubly linked list. */
  gui.file2tag.history = g_list_prepend(gui.file2tag.history, (gchar*)strdup(newtext));
  
  /* If the list becomes too long, remove the oldest item. */
  if (g_list_length(gui.file2tag.history) > 8) {
    item = g_list_last(gui.file2tag.history);
    dead = (gchar*)item->data;
    gui.file2tag.history = g_list_remove_link(gui.file2tag.history, item);
    g_list_free_1(item);
    free(dead);
  }
  
  /* And show them in the GUI. */
  gtk_combo_set_popdown_strings(combo, gui.file2tag.history);
}
/******************************************************************************
 * END STATICS
 ******************************************************************************/


/******************************************************************************
 * CALLBACKS
 ******************************************************************************/
/*
 * The "Save" button callback.
 */
void on_button_file2tag_save_clicked(void)
{
  GtkToggleButton *toggle = NULL;
  JobArg          arg;
  int             i = -1;
  
  /* Update the history format list. */
  gui_file2tag_history_push(gui.file2tag.format);
  
  arg.ruleset = ruleset_new(NULL);
  arg.preview = cfg.preview_file2tag;
  
  g_assert(arg.ruleset != NULL);
  
  statement_append(arg.ruleset, F_FILENAME, COND_ALWAYS, 0, INT16, NULL,
    RULE_FORMAT_GRAB, STR, (void *)gui.file2tag.format, INT16, (void *)NULL);
  
  job_preview_clear();
  gui_queue_selection_for_each(GTK_TREE_VIEW(glade_xml_get_widget(gui.mainwindow_xml, "treeview_queue")), (void*)job_append, &arg);
  engine_wakeup();
  
  if (cfg.preview_file2tag && !gui.preview)
    gui.preview = gui_preview_init((PreviewDestroyFunc)on_preview_destroy);
}


/*
 * The "Filename -> Tag"'s entry-changed callback.
 */
void on_entry_file2tag_format_changed(GtkEntry *entry)
{
  g_assert(entry != NULL);
  
  /* Keep the updated entry text in memory. */
  gui.file2tag.format = (const gchar *)gtk_entry_get_text(entry);
  
  gui_file2tag_update();
  
  /* Activate or deactivate the "save" button. */
  gtk_widget_set_sensitive(
    GTK_WIDGET(glade_xml_get_widget(gui.mainwindow_xml, "button_file2tag_save")),
    (gui.file2tag.format && *gui.masseditor.rulesetname != '\0'));
}
/******************************************************************************
 * END CALLBACKS
 ******************************************************************************/


void gui_file2tag_init(void)
{
  gui.file2tag.history = NULL;
}


void gui_file2tag_update(void)
{
  GtkTreeView       *queue      = NULL;
  Ruleset           *ruleset    = NULL;
  FileInfo          info;
  FileInfo          *selinfo    = NULL;
  GtkLabel          *label      = NULL;
  Pair              pair;
  gchar             filename[2048];
  int               i           = -1;
  
  /* Receive the first selected item from the queue. */
  queue = (GtkTreeView*)glade_xml_get_widget(gui.mainwindow_xml, "treeview_queue");
  if (!(selinfo = (FileInfo*)gui_queue_get_selection_first_data(queue)))
    return;
  
  /* Update the preview. */
  info.tag[0] = (Tag *)calloc(1, sizeof(Tag));
  info.tag[1] = (Tag *)calloc(1, sizeof(Tag));
  if (!strrchr(selinfo->filename, '/'))
    return;
  strncpy(filename, strrchr(selinfo->filename, '/') + 1, 2047);
  rule_format_to_destination(filename, gui.file2tag.format, &info);
  
  /* Walk through the field list, applying the rule to every matching field. */
  while (field_get(++i, info.tag[0], &pair)) {
    label = GTK_LABEL(glade_xml_get_widget(gui.mainwindow_xml, pair.name));
    gtk_label_set_label(label, pair.dest);
  }
}
