/* The Cantus project.
 * (c)2002, 2003 by Samuel Abels (spam debain org)
 * This project's homepage is: http://www.debain.org/cantus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libintl.h>
#include "lib_id3genre.h"
#include "lib_strings.h"
#include "gui_genre.h"
#include "gui_init.h"
#include "gui_tageditor.h"

#define _(String) gettext (String)
#define gettext_noop(String) (String)
#define N_(String) gettext_noop (String)

extern GUI gui;

enum COLUMNS {
  COL_GENRE,
  COL_ID,
  N_COLUMNS
};


/***************************************************************************************
 * STATICS
 ***************************************************************************************/
/***************************************************************************************
 * END STATICS
 ***************************************************************************************/


/***************************************************************************************
 * CALLBACKS
 ***************************************************************************************/
void on_genre_selection_changed(GtkTreeSelection *selection, void *trash)
{
  GtkTreeModel *model = NULL;
  GList *sellist = NULL;
  GList *item = NULL;
  GtkTreeIter iter;
  gchar **genrelist = NULL;
  gchar *curgenre = NULL;
  gchar *pnewgenre = NULL;
  gchar *newgenre = NULL;
  gint i = 0;
  
  /* Get the old genre string and split it into an array. */
  genrelist = explode(gui.tageditor.genre, ",");
  
  /* Get a list of selected items. */
  sellist = gtk_tree_selection_get_selected_rows(selection, &model);
  
  i = -1;
  while (genrelist[++i])
    string_wipe_spaces(genrelist[i], 2);
  
  /* Remove unselected genres from that array. */
  gtk_tree_model_get_iter_first(model, &iter);
  do {
    if (gtk_tree_selection_iter_is_selected(selection, &iter))
      continue;
    
    gtk_tree_model_get(model, &iter, COL_GENRE, &curgenre, -1);
    if (!(curgenre = (gchar *)array_element_case_insensitive((const gchar **)genrelist, (const gchar *)curgenre)))
      continue;
    
    *curgenre = '\0';
  } while (gtk_tree_model_iter_next(model, &iter));
  
  /* Append new selected genres to that array. */
  item = sellist;
  while (item) {
    gtk_tree_model_get_iter(model, &iter, (GtkTreePath *)item->data);
    gtk_tree_model_get(model, &iter, COL_GENRE, &curgenre, -1);
    
    /* Double genre is not allowed. */
    if (array_element_case_insensitive((const gchar **)genrelist, (const gchar *)curgenre)) {
      item = item->next;
      continue;
    }
    
    genrelist[i++] = strdup(curgenre);
  }
  
  /* Merge the array into one string, seperated by commas. */
  i = -1;
  while (genrelist[++i] && *genrelist[i] == '\0');
  if (!genrelist[i]) {
    free_array(genrelist);
    gui_tageditor_genre_set("");
    return;
  }
  
  newgenre = strdup(genrelist[i]);
  while (genrelist[++i]) {
    if (*genrelist[i] == '\0')
      continue;
    
    pnewgenre = newgenre;
    newgenre = g_strconcat(newgenre, ", ", genrelist[i], NULL);
    g_free(pnewgenre);
  }
  
  free_array(genrelist);
  
  gui_tageditor_genre_set((const gchar *)newgenre);
  
  g_free(newgenre);
}


void on_button_genre_close_clicked(GtkButton *button, gpointer trash)
{
  gtk_widget_hide(gui.genre);
}


gint on_genre_delete_event(GtkWidget *widget, GdkEvent *event, gpointer trash)
{
  gtk_widget_hide(widget);
  return(TRUE);
}
/***************************************************************************************
 * END CALLBACKS
 ***************************************************************************************/


void gui_genre_init(void)
{
  GtkTreeView *treeview = NULL;
  GtkTreeViewColumn *column = NULL;
  GtkCellRenderer *renderer = NULL;
  GtkTreeIter iter;
  int i = -1;
  
  /* Build the mainwindow. */
  if (!(gui.genre_xml = glade_xml_new(PACKAGE_DATA_DIR "/cantus-2.glade", "genre", NULL))) {
    fprintf(stderr, "Error: gui_init(): Interface file not found. (%s)", PACKAGE_DATA_DIR "/cantus.glade");
    exit(1);
  }
  
  glade_xml_signal_autoconnect(gui.genre_xml);
  gui.genre = glade_xml_get_widget(gui.genre_xml, "genre");
  
  /* Init the genre list. */
  treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui.genre_xml, "treeview_genre"));
  
  /* Create a model and connect signals. */
  gui.genre_store = gtk_tree_store_new(N_COLUMNS, G_TYPE_STRING, G_TYPE_INT);
  gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), GTK_TREE_MODEL(gui.genre_store));
  
  /* Create a cell render. */
  renderer = gtk_cell_renderer_text_new();
  
  /* Add the columns to the view. */
  column = gtk_tree_view_column_new_with_attributes(_("Genre"), renderer, "text", COL_GENRE, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
  
  /* Selection type & callback. */
  gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview)), GTK_SELECTION_MULTIPLE);
  g_signal_connect(G_OBJECT(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview))), "changed", G_CALLBACK(on_genre_selection_changed), NULL);
  
  while (genre[++i]) {
    gtk_tree_store_append(gui.genre_store, &iter, NULL);
    gtk_tree_store_set(gui.genre_store, &iter, COL_GENRE, genre[i], COL_ID, i, -1);
  }
}


void gui_genre_selection_init(const gchar *text)
{
  GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(glade_xml_get_widget(gui.genre_xml, "treeview_genre")));
  GtkTreeIter iter;
  gchar **genrelist = NULL;
  gint i = -1, f = -1;
  
  if (!text || *text == '\0')
    return;
  
  genrelist = explode(text, ",");
  
  while (genrelist[++i]) {
    string_wipe_spaces(genrelist[i], 2);
    
    f = -1;
    while (genre[++f]) {
      if (strcasecmp(genrelist[i], genre[f]) == 0) {
        gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(gui.genre_store), &iter, NULL, f);
        gtk_tree_selection_select_iter(selection, &iter);
      }
    }
  }
  
  free_array(genrelist);
}

#undef COLUMNS
