/* The Cantus project.
 * (c)2002, 2003 by Samuel Abels (spam debain org)
 * This project's homepage is: http://www.debain.org/cantus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libintl.h>
#include "gui_init.h"
#include "gui_queue.h"
#include "gui_mainwindow.h"
#include "gui_addfiles.h"
#include "gui_preferences.h"
#include "gui_file2tag.h"
#include "engine_init.h"

#define _(String) gettext (String)
#define gettext_noop(String) (String)
#define N_(String) gettext_noop (String)

extern Engine engine;
extern Cfg cfg;
extern GUI gui;

/***************************************************************************************
 * STATICS
 ***************************************************************************************/
/*
 * Updates the GUI.
 */
static int on_gui_sleep_timeout(void *null)
{
  // FIXME: Be nice and don't just exit.
  if (engine.shutdown)
    gtk_main_quit();
  
  /* Lock everything. */
  gdk_threads_enter();
  pthread_mutex_lock(&engine.mut);
  gui_queue_update(GTK_TREE_MODEL(gui.queue));
  gui_preview_update(gui.preview, engine.previewlist);
  /* Unlock stuff. */
  pthread_mutex_unlock(&engine.mut);
  gdk_threads_leave();
  
  gtk_timeout_add(GUI_UPDATE_INTERVAL, on_gui_sleep_timeout, NULL);
  return(0);
}
/***************************************************************************************
 * END STATICS
 ***************************************************************************************/

/*
 * Initialize the GUI at program start.
 */
void gui_init(void)
{
  glade_init();
  
  /* Default options. */
  gui.showhidden  = FALSE;
  gui.working     = FALSE;
  *gui.defaultdir = '\0';
  cfg.save_rulesets_on_exit = TRUE;
  
  /* Load options from the configfile. */
  // FIXME: Check compatibility level.
  preferences_load();
  
  if (*gui.defaultdir == '\0')
    strncpy(gui.defaultdir, getenv("HOME"), 2047);
  
  /* Initialize windows. */
  gui.addfiles.widget = NULL;
  gui.genre           = NULL;
  gui.preferences     = NULL;
  gui.preview         = NULL;
  gui_mainwindow_create();
  gui_file2tag_init();
  
  /* Add a timeout to update the GUI again. */
  gtk_timeout_add(GUI_UPDATE_INTERVAL, on_gui_sleep_timeout, NULL);
}


void gui_destroy(void)
{
  gui_addfiles_destroy();
  gui_mainwindow_destroy();
  gui_preferences_destroy();
}
