/* The Cantus project.
 * (c)2002, 2003 by Samuel Abels (spam debain org)
 * This project's homepage is: http://www.debain.org/cantus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <stdio.h>
#include <stdlib.h>
#include <libintl.h>
#include "gui_init.h"
#include "gui_queue.h"
#include "gui_progress.h"
#include "gui_preferences.h"
#include "engine_queue.h"
#include "gui_addfiles.h"
#include "gui_tageditor.h"
#include "gui_masseditor.h"
#include "gui_genre.h"
#include "engine_init.h"
#include "engine_queue.h"
#include "engine_ruleset.h"

#define _(String) gettext (String)
#define gettext_noop(String) (String)
#define N_(String) gettext_noop (String)

extern Engine engine;
extern Cfg cfg;
extern GUI gui;

/***************************************************************************************
 * MENU CALLBACKS
 ***************************************************************************************/
void on_menu_file_add_activate(GtkMenuItem *menuitem, gpointer trash)
{
  if (!gui.addfiles.widget)
    gui_addfiles_create();
  gtk_widget_show(gui.addfiles.widget);
}


void on_menu_file_remove_activate(GtkMenuItem *menuitem, gpointer trash)
{
  gui_queue_selection_for_each(
    GTK_TREE_VIEW(glade_xml_get_widget(gui.mainwindow_xml, "treeview_queue")),
    queue_remove,
    NULL);
  gui_queue_update(GTK_TREE_MODEL(gui.queue));
}


void on_menu_file_quit_activate(GtkMenuItem *menuitem, gpointer trash)
{
  engine_stop();
  gtk_main_quit();
}


void on_menu_edit_preferences_activate(GtkMenuItem *menuitem, gpointer trash)
{
  if (!gui.preferences) {
    gui_preferences_init();
    gui_preferences_activate_tab(1);
  }
  gtk_widget_show(gui.preferences);
}


void on_menu_edit_rulesets_save_activate(GtkMenuItem *menuitem, gpointer trash)
{
  /* Save all rulesets to a configfile. */
  rulesetlist_save(engine.rulesetlist);
}


void on_menu_help_about_activate(GtkMenuItem *menuitem, gpointer trash)
{
  if (!gui.preferences)
    gui_preferences_init();
  gui_preferences_activate_tab(0);
  gtk_widget_show(gui.preferences);
}
/***************************************************************************************
 * END MENU CALLBACKS
 ***************************************************************************************/


/***************************************************************************************
 * TOOLBAR CALLBACKS
 ***************************************************************************************/
void on_button_clear_clicked(GtkButton *button, gpointer trash)
{
  queue_clear();
  gui_queue_update(GTK_TREE_MODEL(gui.queue));
}


void on_button_addfiles_clicked(GtkButton *button, gpointer trash)
{
  if (!gui.addfiles.widget)
    gui_addfiles_create();
  gtk_widget_show(gui.addfiles.widget);
}


void on_button_remove_clicked(GtkButton *button, gpointer trash)
{
  gui_queue_selection_for_each(
    GTK_TREE_VIEW(glade_xml_get_widget(gui.mainwindow_xml, "treeview_queue")),
    queue_remove,
    NULL);
  gui_queue_update(GTK_TREE_MODEL(gui.queue));
}


void on_button_preferences_clicked(GtkButton *button, gpointer trash)
{
  if (!gui.preferences) {
    gui_preferences_init();
    gui_preferences_activate_tab(1);
  }
  gtk_widget_show(gui.preferences);
}


void on_button_about_clicked(GtkButton *button, gpointer trash)
{
  if (!gui.preferences)
    gui_preferences_init();
  gui_preferences_activate_tab(0);
  gtk_widget_show(gui.preferences);
}


void on_button_close_clicked(GtkButton *button, gpointer trash)
{
  engine_stop();
  gtk_main_quit();
}
/***************************************************************************************
 * END TOOLBAR CALLBACKS
 ***************************************************************************************/


/***************************************************************************************
 * OTHER CALLBACKS
 ***************************************************************************************/
/*
 * Updates the progressbar.
 */
static int on_progressbar_sleep_timeout(void *null)
{
  gdk_threads_enter();
  pthread_mutex_lock(&engine.mut);
  gui_progress_update(GTK_PROGRESS_BAR(gui.progressbar));
  pthread_mutex_unlock(&engine.mut);
  gdk_threads_leave();
  gtk_timeout_add(PROGRESS_UPDATE_INTERVAL, on_progressbar_sleep_timeout, NULL);
  return (0);
}
/***************************************************************************************
 * END OTHER CALLBACKS
 ***************************************************************************************/


/*
 * Initialize the GUI at program start.
 */
void gui_mainwindow_create(void)
{
  GtkWidget *treeview = NULL;
  
  /* Build the mainwindow. */
  if (!(gui.mainwindow_xml = glade_xml_new(PACKAGE_DATA_DIR "/cantus-2.glade", "mainwindow", NULL))) {
    fprintf(stderr, "Error: gui_init(): Interface file not found. (%s)", PACKAGE_DATA_DIR "/cantus-2.glade");
    exit(1);
  }
  
  glade_xml_signal_autoconnect(gui.mainwindow_xml);
  gui.mainwindow = glade_xml_get_widget(gui.mainwindow_xml, "mainwindow");
  
  /* Init the queue. */
  treeview = glade_xml_get_widget(gui.mainwindow_xml, "treeview_queue");
  gui.queue = gui_queue_create(GTK_TREE_VIEW(treeview));
  
  gui_tageditor_init();
  gui_progress_init();
  gui_masseditor_init(&gui.masseditor);
  gui_masseditor_rulesetlist_update();
  
  gtk_timeout_add(PROGRESS_UPDATE_INTERVAL, on_progressbar_sleep_timeout, NULL);
}


void gui_mainwindow_destroy(void)
{
  gtk_widget_destroy(gui.mainwindow);
}
