/* The Cantus project.
 * (c)2002, 2003 by Samuel Abels (spam debain org)
 * This project's homepage is: http://www.debain.org/cantus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <libintl.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "gui_init.h"
#include "gui_masseditor.h"
#include "gui_ruleeditor.h"
#include "lib_configfile.h"
#include "engine_init.h"
#include "engine_jobs.h"

#define _(String) gettext (String)
#define gettext_noop(String) (String)
#define N_(String) gettext_noop (String)

extern GUI    gui;
extern Cfg    cfg;
extern Engine engine;

static gchar configfilename[2048];


/******************************************************************************
 * STATICS
 ******************************************************************************/
static void set_configfilename(void)
{
  snprintf(configfilename, 2047, "%s/.cantusrules", getenv("HOME"));
}


static PreviewDestroyFunc on_preview_destroy(void)
{
  gui.preview = NULL;
}
/******************************************************************************
 * END STATICS
 ******************************************************************************/


/******************************************************************************
 * CALLBACKS
 ******************************************************************************/
void on_button_masseditor_execute_clicked(void)
{
  JobArg arg;
  
  arg.ruleset = ruleset_find(engine.rulesetlist, gui.masseditor.rulesetname);
  arg.preview = cfg.preview_masseditor;
  
  g_assert(arg.ruleset != NULL);
  
  job_preview_clear();
  gui_queue_selection_for_each(GTK_TREE_VIEW(glade_xml_get_widget(gui.mainwindow_xml, "treeview_queue")), (void*)job_append, &arg);
  engine_wakeup();
  
  if (cfg.preview_masseditor && !gui.preview)
    gui.preview = gui_preview_init((PreviewDestroyFunc)on_preview_destroy);
}


void on_button_masseditor_new_clicked(void)
{
  // FIXME: Generate a rulesetname that does not yet exist.
  gui_ruleeditor_init(ruleset_new(_("New Ruleset")), NULL);
  gtk_widget_show(gui.ruleeditor.ruleeditor);
}


void on_button_masseditor_edit_clicked(void)
{
  Ruleset *ruleset = ruleset_find(engine.rulesetlist, gui.masseditor.rulesetname);
  
  gui_ruleeditor_init(ruleset_copy(ruleset), ruleset);
  gtk_widget_show(gui.ruleeditor.ruleeditor);
}


void on_button_masseditor_delete_clicked(void)
{
  GtkTreeView *treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui.mainwindow_xml, "treeview_masseditor_ruleset"));
  Ruleset *ruleset = ruleset_find(engine.rulesetlist, gui.masseditor.rulesetname);
  
  g_assert(treeview != NULL);
  g_assert(ruleset != NULL);
  
  engine.rulesetlist = g_list_remove(engine.rulesetlist, ruleset);
  ruleset_unref(ruleset);
  
  /* Update the statement list. */
  gui_masseditor_rulesetlist_update();
  ruleset = ruleset_find(engine.rulesetlist, gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(gui.mainwindow_xml, "entry_masseditor_ruleset"))));
  gui_statementlist_update(gtk_tree_view_get_model(treeview), ruleset);
}


void on_entry_masseditor_ruleset_changed(void)
{
  GtkTreeView *treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui.mainwindow_xml, "treeview_masseditor_ruleset"));
  Ruleset *ruleset = NULL;
  
  g_assert(treeview != NULL);
  
  /* Keep the updated entry text in memory. */
  gui.masseditor.rulesetname = (const gchar *)gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(gui.mainwindow_xml, "entry_masseditor_ruleset")));
  
  /* Activate or deactivate the "Edit" and "Delete" buttons. */
  gtk_widget_set_sensitive(
    GTK_WIDGET(glade_xml_get_widget(gui.mainwindow_xml, "button_masseditor_edit")),
    (gui.masseditor.rulesetname && *gui.masseditor.rulesetname != '\0'));
  gtk_widget_set_sensitive(
    GTK_WIDGET(glade_xml_get_widget(gui.mainwindow_xml, "button_masseditor_delete")),
    (gui.masseditor.rulesetname && *gui.masseditor.rulesetname != '\0'));
  
  /* Find the corresponding ruleset. */
  if (!(ruleset = ruleset_find(engine.rulesetlist, gui.masseditor.rulesetname)))
    return;
  
  /* Update the ruleset and statement list. */
  gui_statementlist_update(gtk_tree_view_get_model(treeview), ruleset);
}
/******************************************************************************
 * END CALLBACKS
 ******************************************************************************/


/*
 * Masseditor first-time initialisation on program startup.
 */
void gui_masseditor_init(Masseditor *me)
{
  me->rulesetname = NULL;
  gui_statementlist_init(GTK_TREE_VIEW(glade_xml_get_widget(gui.mainwindow_xml, "treeview_masseditor_ruleset")));
}


/*
 * Update the list of available rulesets in the GUI (which is a dropdown menu).
 */
void gui_masseditor_rulesetlist_update(void)
{
  GtkCombo  *combo = GTK_COMBO(glade_xml_get_widget(gui.mainwindow_xml, "combo_masseditor_ruleset"));
  GList     *names    = NULL;
  GList     *item     = NULL;
  int i = -1;
  
  g_assert(combo != NULL);
  
  /* Put the ruleset names into a doubly linked list. */
  item = engine.rulesetlist;
  while (item) {
    names = g_list_append(names, ((Ruleset*)item->data)->name);
    item = item->next;
  }
  
  if (!names) {
    names = g_list_append(names, NULL);
    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui.mainwindow_xml, "entry_masseditor_ruleset")), "");
  }
  
  /* And show them in the GUI (sorted). */
  names = g_list_sort(names, (GCompareFunc)strcasecmp);
  gtk_combo_set_popdown_strings(combo, names);
  /* This is a hack to work around a bug in some versions of GTK. */
  gtk_combo_set_popdown_strings(combo, names);
  
  /* Cleanups. */
  g_list_free(names);
}


/*
 * Select the given ruleset in the ruleset dropdown menu.
 */
void gui_masseditor_ruleset_select(Ruleset *rs)
{
  if (rs)
    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui.mainwindow_xml, "entry_masseditor_ruleset")), rs->name);
}
