/* The Cantus project.
 * (c)2002, 2003 by Samuel Abels (spam debain org)
 * This project's homepage is: http://www.debain.org/cantus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <libintl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gui_preview.h"
#include "engine_jobs.h"

#define _(String) gettext (String)
#define gettext_noop(String) (String)
#define N_(String) gettext_noop (String)

enum COLUMNS {
  COL_ENABLE,
  COL_DIRECTORY,
  COL_FILENAME_OLD,
  COL_FILENAME_NEW,
  COL_ARTIST,
  COL_ALBUM,
  COL_SONG,
  COL_TRACK,
  COL_YEAR,
  COL_GENRE,
  COL_COMMENT,
  COL_DATA,
  N_COLUMNS
};

typedef struct {
  int   col;
  gchar *name;
} Pair;

static Pair colmap[] = {
  { COL_ENABLE,       N_("Rename") },
  { COL_DIRECTORY,    N_("Directory") },
  { COL_FILENAME_OLD, N_("Current Filename") },
  { COL_FILENAME_NEW, N_("New Filename") },
  { COL_ARTIST,       N_("Artist") },
  { COL_ALBUM,        N_("Album") },
  { COL_SONG,         N_("Song") },
  { COL_TRACK,        N_("Track") },
  { COL_YEAR,         N_("Year") },
  { COL_GENRE,        N_("Genre") },
  { COL_COMMENT,      N_("Comment") },
  { 0, NULL }
};


/***************************************************************************************
 * STATICS
 ***************************************************************************************/
typedef struct {
  void        *info;
  int         column;
  GtkTreeIter *iter;
  gboolean    found;
} ForEachCmpData;

static gboolean pointer_same(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, ForEachCmpData *rowdata)
{
  void *data = NULL;
  
  gtk_tree_model_get(model, iter, rowdata->column, &data, -1);
  
  if (rowdata->info == data) {
    memcpy(rowdata->iter, iter, sizeof(GtkTreeIter));
    rowdata->found = TRUE;
    return(TRUE);
  }
  
  return(FALSE);
}


static gboolean gui_preview_job_remove(GtkListStore *store, GtkTreePath *path, GtkTreeIter *iter, void **args)
{
  Job   *job        = NULL;
  GList *joblist    = args[0];
  GList **iterlist  = args[1];
  
  /* Get the current GUI item. */
  gtk_tree_model_get(GTK_TREE_MODEL(store), iter, COL_DATA, &job, -1);
  g_assert(job != NULL);
  
  /* If the item is still in the list, do nothing. */
  if (g_list_find(joblist, job))
    return(FALSE);
  
  /* Otherwise store the iter of this item in a list for later removal. */
  *iterlist = g_list_append(*iterlist, gtk_tree_iter_copy(iter));
  
  return(FALSE);
}


/*
 * Find the tree row/column that contains the file "info"
 * and store the position in "iter".
 * Returns TRUE on success, otherwise FALSE.
 */
static gboolean find_treeview_row(GtkTreeModel *model, GtkTreeIter * iter, int column, void * info)
{
  ForEachCmpData data;
  
  data.info   = info;
  data.column = column;
  data.iter   = iter;
  data.found  = FALSE;
  
  gtk_tree_model_foreach(model, (GtkTreeModelForeachFunc)pointer_same, &data);
  
  return(data.found);
}


static GHFunc gui_preview_job_add(Job *job, GtkListStore *store)
{
  GtkTreeIter iter;
  gchar       directory[2048];
  gchar       oldfilename[2048];
  gchar       newfilename[2048];
  
  g_assert(job != NULL);
  
  if (!job->updated)
    return(NULL);
  
  /* Create the directory name. */
  strncpy(directory, job->info->filename, 2047);
  if (strrchr(directory, '/'))
    *(strrchr(directory, '/') + 1) = '\0';
  
  /* Create the old filename. */
  if (strrchr(job->info->filename, '/'))
    strncpy(oldfilename, strrchr(job->info->filename, '/') + 1, 2047);
  else
    strncpy(oldfilename, job->info->filename, 2047);
  
  /* Create the new filename. */
  if (strrchr(job->destinfo->filename, '/'))
    strncpy(newfilename, strrchr(job->destinfo->filename, '/') + 1, 2047);
  else
    strncpy(newfilename, job->destinfo->filename, 2047);
  
  /* If it's not already in the list, add it. */
  if (!find_treeview_row(GTK_TREE_MODEL(store), &iter, COL_DATA, job)) {
    gtk_list_store_append(store, &iter);
    gtk_list_store_set(store, &iter, COL_ENABLE, TRUE, -1);
  }
  
  /* Eighter way, update it. */
  gtk_list_store_set(store, &iter,
    COL_DIRECTORY,    directory,
    COL_FILENAME_OLD, oldfilename,
    COL_FILENAME_NEW, newfilename,
    COL_ARTIST,       job->destinfo->tag[1] ? job->destinfo->tag[1]->artist   : NULL,
    COL_ALBUM,        job->destinfo->tag[1] ? job->destinfo->tag[1]->album    : NULL,
    COL_SONG,         job->destinfo->tag[1] ? job->destinfo->tag[1]->title    : NULL,
    COL_TRACK,        job->destinfo->tag[1] ? job->destinfo->tag[1]->track    : NULL,
    COL_YEAR,         job->destinfo->tag[1] ? job->destinfo->tag[1]->year     : NULL,
    COL_GENRE,        job->destinfo->tag[1] ? job->destinfo->tag[1]->genre    : NULL,
    COL_COMMENT,      job->destinfo->tag[1] ? job->destinfo->tag[1]->comment  : NULL,
    COL_DATA,         job,
    -1);
  
  job->updated = FALSE;
  
  return(NULL);
}


static void preview_row_toggled(GtkCellRendererToggle *cell, gchar *path_str, GtkListStore *store)
{
  GtkTreePath *path = gtk_tree_path_new_from_string(path_str);
  gboolean    enabled = FALSE;
  GtkTreeIter iter;
  
  if (!gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &iter, path))
    return;
  gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, COL_ENABLE, &enabled, -1);
  gtk_list_store_set(store, &iter, COL_ENABLE, !enabled, -1);
}


/*
 * Initialize the preview list.
 */
static GtkListStore *gui_preview_list_create(GtkTreeView *treeview)
{
  GtkListStore      *store    = NULL;
  GtkTreeViewColumn *column   = NULL;
  GtkCellRenderer   *renderer = NULL;
  int               i         = 0;

  g_assert(treeview != NULL);
  
  /* Create a model and connect signals. */
  store = gtk_list_store_new(N_COLUMNS,
    G_TYPE_BOOLEAN,
    G_TYPE_STRING,
    G_TYPE_STRING,
    G_TYPE_STRING,
    G_TYPE_STRING,
    G_TYPE_STRING,
    G_TYPE_STRING,
    G_TYPE_STRING,
    G_TYPE_STRING,
    G_TYPE_STRING,
    G_TYPE_STRING,
    G_TYPE_POINTER);
  gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), GTK_TREE_MODEL(store));
  
  /* Create a cell renderer for the first column, which is a toggle, and add it to the view. */
  renderer = gtk_cell_renderer_toggle_new();
  column = gtk_tree_view_column_new_with_attributes(_(colmap[COL_ENABLE].name), renderer, "active", COL_ENABLE, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
  gtk_tree_view_column_set_sort_column_id(column, COL_ENABLE);
  g_signal_connect(G_OBJECT(renderer), "toggled", G_CALLBACK(preview_row_toggled), store);
  
  /* Create a cell renderer for all other columns. */
  renderer = gtk_cell_renderer_text_new();
  
  /* Add the columns to the view. */
  for (i = 1; i < N_COLUMNS - 1; i++) {
    column = gtk_tree_view_column_new_with_attributes(_(colmap[i].name), renderer, "text", i, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_column_set_sort_column_id(column, i);
  }
  
  /* Selection type. */
  gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview)), GTK_SELECTION_MULTIPLE);
  
  return(store);
}


static gboolean job_remove_if_not_checked(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, void *data)
{
  gboolean  checked = FALSE;
  Job       *job    = NULL;
  
  gtk_tree_model_get(model, iter, COL_ENABLE, &checked, COL_DATA, &job, -1);
  g_assert(job != NULL);
  
  if (checked)
    return(FALSE);
  
  job_remove_from_preview(job);
  
  return(FALSE);
}
/***************************************************************************************
 * END STATICS
 ***************************************************************************************/


/***************************************************************************************
 * CALLBACKS
 ***************************************************************************************/
void on_button_preview_cancel_clicked(GtkButton *button, gpointer trash)
{
  job_preview_clear();
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}


void on_button_preview_execute_clicked(GtkButton *button, Preview *preview)
{
  g_assert(preview != NULL);
  
  /* Remove non-checked items from the list. */
  gtk_tree_model_foreach(GTK_TREE_MODEL(preview->liststore), (GtkTreeModelForeachFunc)job_remove_if_not_checked, NULL);
  
  /* Execute the changes. */
  job_previewlist_execute();
  
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}
/***************************************************************************************
 * END CALLBACKS
 ***************************************************************************************/


Preview *gui_preview_init(PreviewDestroyFunc on_destroy)
{
  Preview   *preview  = calloc(1, sizeof(Preview));
  GtkButton *button   = NULL;
  gint    i           = -1;
  
  /* Build the mainwindow. */
  if (!(preview->xml = glade_xml_new(PACKAGE_DATA_DIR "/cantus-2.glade", "preview", NULL))) {
    fprintf(stderr, "Error: gui_init(): Interface file not found. (%s)", PACKAGE_DATA_DIR "/cantus.glade");
    exit(1);
  }
  
  glade_xml_signal_autoconnect(preview->xml);
  preview->widget = glade_xml_get_widget(preview->xml, "preview");
  button = GTK_BUTTON(glade_xml_get_widget(preview->xml, "button_preview_execute"));
  
  /* Init the genre list. */
  preview->treeview = GTK_TREE_VIEW(glade_xml_get_widget(preview->xml, "treeview_preview"));
  
  /* Create a model and connect signals. */
  preview->liststore  = gui_preview_list_create(preview->treeview);
  preview->on_destroy = on_destroy;
  g_signal_connect(G_OBJECT(preview->widget), "destroy", G_CALLBACK(gui_preview_destroy), preview);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(on_button_preview_execute_clicked), preview);
  
  return(preview);
}


void gui_preview_update(Preview *preview, GList *joblist)
{
  GtkTreeIter iter;
  GList       *iterlist = NULL;
  GList       *item     = NULL;
  void        *args[5];
  
  if (!preview)
    return;
  
  g_assert(preview->liststore != NULL);
  
  args[0] = joblist;
  args[1] = &iterlist;
  
  /* Remove dead items from the GUI list. */
  if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(preview->liststore), &iter)) {
    gtk_tree_model_foreach(GTK_TREE_MODEL(preview->liststore), (GtkTreeModelForeachFunc)gui_preview_job_remove, &args);
    item = iterlist;
    while (item) {
      gtk_list_store_remove(GTK_LIST_STORE(preview->liststore), (GtkTreeIter*)item->data);
      gtk_tree_iter_free((GtkTreeIter*)item->data);
      item = item->next;
    }
    g_list_free(iterlist);
  }
  
  /* If the list is empty, we're done. */
  if (!joblist)
    return;
  
  /* Now add all new queue items to the GUI. */
  g_list_foreach(joblist, (GFunc)gui_preview_job_add, preview->liststore);
}


void gui_preview_destroy(GtkWidget *widget, Preview *preview)
{
  if (!preview)
    return;
  preview->on_destroy();
  g_free(preview);
}

#undef COLUMNS
