/* The Cantus project.
 * (c)2002, 2003 by Samuel Abels (spam debain org)
 * This project's homepage is: http://www.debain.org/cantus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <libintl.h>
#include <pthread.h>
#include <stdio.h>
#include <glade/glade.h>
#include "gui_init.h"
#include "gui_ruleeditor.h"
#include "gui_statementlist.h"
#include "gui_lib_popup.h"
#include "engine_init.h"
#include "engine_rules.h"
#include "engine_ruleset.h"

#define _(String) gettext (String)
#define gettext_noop(String) (String)
#define N_(String) gettext_noop (String)

extern Engine engine;
extern GUI    gui;

enum TYPES {
  TOGGLE,
  ENTRY,
  OPTIONMENU
};

typedef struct {
  gchar *widgetname;
  gint  type;
  void  *value;
} Map;

static Map options[] = {
  /* Condition. */
  { "optionmenu_ruleeditor_condition",                OPTIONMENU, &gui.ruleeditor.cond_type },
  { "entry_ruleeditor_condition_text",                ENTRY,      &gui.ruleeditor.cond_text },
  { "optionmenu_ruleeditor_condition_field",          OPTIONMENU, &gui.ruleeditor.cond_field },
  
  /* Flags. */
  { "check_ruleeditor_rule_strip_multiple_spaces",    TOGGLE,     &gui.ruleeditor.strip_multiple_spaces },
  { "check_ruleeditor_rule_remove_leading_spaces",    TOGGLE,     &gui.ruleeditor.remove_leading_spaces },
  { "check_ruleeditor_rule_remove_trailing_spaces",   TOGGLE,     &gui.ruleeditor.remove_trailing_spaces },
  { "check_ruleeditor_rule_remove_before_number",     TOGGLE,     &gui.ruleeditor.remove_before_number },
  
  /* Capitol Letters. */
  { "check_ruleeditor_rule_uppercase",                TOGGLE,     &gui.ruleeditor.uppercase },
  { "check_ruleeditor_rule_uppercase_word",           TOGGLE,     &gui.ruleeditor.uppercase_word },
  { "check_ruleeditor_rule_uppercase_first_word",     TOGGLE,     &gui.ruleeditor.uppercase_first_word },
  { "check_ruleeditor_rule_lowercase",                TOGGLE,     &gui.ruleeditor.lowercase },
  
  /* Static. */
  { "check_ruleeditor_rule_set",                      TOGGLE,     &gui.ruleeditor.set },
  { "entry_ruleeditor_rule_set_to",                   ENTRY,      &gui.ruleeditor.set_to },
  
  /* String modifiers. */
  { "check_ruleeditor_rule_replace",                  TOGGLE,     &gui.ruleeditor.replace },
  { "entry_ruleeditor_rule_replace_from",             ENTRY,      &gui.ruleeditor.replace_from },
  { "entry_ruleeditor_rule_replace_to",               ENTRY,      &gui.ruleeditor.replace_to },
  
  { "check_ruleeditor_replace_caseinsensitive",       TOGGLE,     &gui.ruleeditor.replace_ci },
  { "entry_ruleeditor_replace_caseinsensitive_from",  ENTRY,      &gui.ruleeditor.replace_ci_from },
  { "entry_ruleeditor_replace_caseinsensitive_to",    ENTRY,      &gui.ruleeditor.replace_ci_to },
  
  /* Buffer Operations. */
  { "check_ruleeditor_rule_buffer_grab",              TOGGLE,     &gui.ruleeditor.buffer_grab },
  { "entry_ruleeditor_rule_buffer_grab_format",       ENTRY,      &gui.ruleeditor.buffer_grab_format },
  { "check_ruleeditor_rule_buffer_copy_to",           TOGGLE,     &gui.ruleeditor.buffer_copy_to },
  
  /* Destination. */
  { "check_ruleeditor_apply_artist",                  TOGGLE,     &gui.ruleeditor.dest_artist },
  { "check_ruleeditor_apply_song",                    TOGGLE,     &gui.ruleeditor.dest_song },
  { "check_ruleeditor_apply_album",                   TOGGLE,     &gui.ruleeditor.dest_album },
  { "check_ruleeditor_apply_track",                   TOGGLE,     &gui.ruleeditor.dest_track },
  { "check_ruleeditor_apply_year",                    TOGGLE,     &gui.ruleeditor.dest_year },
  { "check_ruleeditor_apply_genre",                   TOGGLE,     &gui.ruleeditor.dest_genre },
  { "check_ruleeditor_apply_comment",                 TOGGLE,     &gui.ruleeditor.dest_comment },
  { "check_ruleeditor_apply_filename",                TOGGLE,     &gui.ruleeditor.dest_filename },
  
  { NULL, 0, NULL }
};


/***************************************************************************************
 * STATICS
 ***************************************************************************************/
/*
 * Build the ruleeditor using libglade.
 */
static void gui_ruleeditor_build(void)
{
  gint i = -1;
  
  /* Build the window. */
  if (!(gui.ruleeditor.ruleeditor_xml = glade_xml_new(PACKAGE_DATA_DIR "/cantus-2.glade", "ruleeditor", NULL))) {
    fprintf(stderr, "Error: gui_ruleeditor_init(): Interface file not found. (%s)", PACKAGE_DATA_DIR "/cantus.glade");
    exit(1);
  }
  g_assert(gui.ruleeditor.ruleeditor_xml != NULL);
  
  gui.ruleeditor.ruleeditor = glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, "ruleeditor");
  g_assert(gui.ruleeditor.ruleeditor != NULL);
  glade_xml_signal_autoconnect(gui.ruleeditor.ruleeditor_xml);
  
  /* Build the statementlist. */
  gui_statementlist_init(GTK_TREE_VIEW(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, "treeview_ruleeditor_rules")), NULL);
  
  /* Get the options. */
  while (options[++i].widgetname) {
    switch (options[i].type) {
    case TOGGLE:
      *(gint *)options[i].value = gtk_toggle_button_get_active(
        GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, options[i].widgetname)));
      break;
    
    case ENTRY:
      *(const gchar **)options[i].value = (const gchar *)gtk_entry_get_text(
        GTK_ENTRY(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, options[i].widgetname)));
      break;
    
    case OPTIONMENU:
      *(gint *)options[i].value = gtk_option_menu_get_history(
        GTK_OPTION_MENU(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, options[i].widgetname)));
      break;
    
    default:
      g_assert_not_reached();
      break;
    }
  }
}


unsigned short int ruleeditor_destination_get(void)
{
  return(
    gui.ruleeditor.dest_filename
    | gui.ruleeditor.dest_artist  << 1
    | gui.ruleeditor.dest_song    << 2
    | gui.ruleeditor.dest_album   << 3
    | gui.ruleeditor.dest_track   << 4
    | gui.ruleeditor.dest_year    << 5
    | gui.ruleeditor.dest_genre   << 6
    | gui.ruleeditor.dest_comment << 7);
}
/***************************************************************************************
 * END STATICS
 ***************************************************************************************/


/***************************************************************************************
 * CALLBACKS
 ***************************************************************************************/
void on_entry_ruleeditor_rulesetname_changed(void)
{
  GtkEntry *entry = GTK_ENTRY(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, "entry_ruleeditor_rulesetname"));
  g_assert(entry != NULL);
  strcpy(gui.ruleeditor.curruleset->name, gtk_entry_get_text(entry));
}


void on_button_ruleeditor_cancel_clicked(void)
{
  on_ruleeditor_delete_event();
}


int on_ruleeditor_delete_event(void)
{
  ruleset_unref(gui.ruleeditor.curruleset);
  gui.ruleeditor.curruleset = NULL;
  gui_ruleeditor_destroy();
  return(TRUE);
}


/*
 * Commit the changes done by the ruleeditor.
 */
void on_button_ruleeditor_ok_clicked(void)
{
  GtkTreeView   *tree_me  = GTK_TREE_VIEW(glade_xml_get_widget(gui.mainwindow_xml, "treeview_masseditor_ruleset"));
  GtkTreeView   *tree_re  = GTK_TREE_VIEW(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, "treeview_ruleeditor_rules"));
  GtkListStore  *store    = GTK_LIST_STORE(gtk_tree_view_get_model(tree_me));
  GList *smlist = NULL;
  
  /* Check if a statement name has been entered. */
  if (!gui.ruleeditor.curruleset->name || *gui.ruleeditor.curruleset->name == '\0') {
    gui_popup(gui.ruleeditor.ruleeditor, _("No ruleset name has been entered.\n\n"
      "Please enter a ruleset name and try again."), GTK_MESSAGE_WARNING);
    return;
  }
  
  smlist = gui_statementlist_order_get(GTK_LIST_STORE(gtk_tree_view_get_model(tree_re)));
  g_list_free(gui.ruleeditor.curruleset->statements);
  gui.ruleeditor.curruleset->statements = smlist;
  
  /* If this is a new ruleset, make sure that the name does not yet exist. */
  if (!gui.ruleeditor.editingruleset) {
    if (ruleset_find(engine.rulesetlist, gui.ruleeditor.curruleset->name)) {
      gui_popup(gui.mainwindow, _("Sorry, but the given ruleset name does already exist.\n\n"
        "Please enter a new ruleset name and try again."), GTK_MESSAGE_WARNING);
      return;
    }
  }
  /* So we are editing an existing ruleset, make sure that the old ruleset exists and that the new name does not. */
  else {
    if (!ruleset_find(engine.rulesetlist, gui.ruleeditor.editingruleset->name)) {
      gui_popup(gui.mainwindow, _("Fatal Error: Existing ruleset not found.\n\n"
        "Please report a bug to the program author."), GTK_MESSAGE_ERROR);
      return;
    }
    
    if (ruleset_find(engine.rulesetlist, gui.ruleeditor.curruleset->name) != NULL
      && ruleset_find(engine.rulesetlist, gui.ruleeditor.curruleset->name) != gui.ruleeditor.editingruleset) {
      gui_popup(gui.mainwindow, _("Sorry, but the given ruleset name does already exist.\n\n"
        "Please enter a new ruleset name and try again."), GTK_MESSAGE_WARNING);
      return;
    }
    
    /* Remove the old item. */
    engine.rulesetlist = g_list_remove(engine.rulesetlist, gui.ruleeditor.editingruleset);
    ruleset_unref(gui.ruleeditor.editingruleset);
  }
  
  /* Add a new item. */
  pthread_mutex_lock(&engine.mut);
  engine.rulesetlist = g_list_append(engine.rulesetlist, gui.ruleeditor.curruleset);
  pthread_mutex_unlock(&engine.mut);
  gui_ruleeditor_destroy();
  
  /* Update the masseditor view. */
  gui_statementlist_update(store, gui.ruleeditor.curruleset);
  gui_masseditor_rulesetlist_update();
  gui_masseditor_ruleset_select(gui.ruleeditor.curruleset);
}



void on_button_ruleeditor_rule_add_clicked(void)
{
  GtkTreeView   *tree   = GTK_TREE_VIEW(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, "treeview_ruleeditor_rules"));
  GtkListStore  *store  = GTK_LIST_STORE(gtk_tree_view_get_model(tree));
  
  g_assert(store != NULL);
  
  gui_ruleeditor_rule_add(gui.ruleeditor.curruleset);
  gui_statementlist_update(store, gui.ruleeditor.curruleset);
}


/*
 * Unused, at the moment!
 */
void on_button_ruleeditor_rule_clear_clicked(void)
{
  GtkTreeView   *tree   = GTK_TREE_VIEW(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, "treeview_ruleeditor_rules"));
  GtkListStore  *store  = GTK_LIST_STORE(gtk_tree_view_get_model(tree));
  
  g_assert(store != NULL);
  g_assert(gui.ruleeditor.curruleset != NULL);
  
  ruleset_clear(gui.ruleeditor.curruleset);
  
  /* Update the GUI. */
  gui_statementlist_update(store, gui.ruleeditor.curruleset);
}


/*
 * Reset all checkboxes and entry fields to their initial state.
 */
void on_button_ruleeditor_rule_reset_clicked(void)
{
  gint i = -1;
  
  while (options[++i].widgetname) {
    switch (options[i].type) {
    case TOGGLE:
      gtk_toggle_button_set_active(
        GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, options[i].widgetname)), FALSE);
      break;
    
    case ENTRY:
      gtk_entry_set_text(
        GTK_ENTRY(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, options[i].widgetname)), "");
      break;
    
    case OPTIONMENU:
      gtk_option_menu_set_history(
        GTK_OPTION_MENU(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, options[i].widgetname)), 0);
      break;
    
    default:
      g_assert_not_reached();
      break;
    }
  }
}


/*
 * Remove all currently selected items from the statement list.
 */
void on_button_ruleeditor_delete_clicked(void)
{
  GtkTreeView       *tree       = GTK_TREE_VIEW(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, "treeview_ruleeditor_rules"));
  GtkListStore      *store      = GTK_LIST_STORE(gtk_tree_view_get_model(tree));
  GtkTreeModel      *model      = GTK_TREE_MODEL(store);
  GtkTreeSelection  *selection  = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
  RuleStatement     *sm         = NULL;
  GList             *sellist    = NULL;
  GtkTreeIter       iter;
  
  /* Get a path list of all selected items in the statement list. */
  if (!selection)
    return;
  sellist = gtk_tree_selection_get_selected_rows(selection, &model);
  
  /* Walk through all selected items and remove them from the list. */
  while (sellist) {
    if (!gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &iter, (GtkTreePath*)sellist->data))
      return;
    gui_statementlist_data_get_from_iter(store, &iter, &sm);
    
    if (!sm) {
      fprintf(stderr, "Fatal Error: on_button_ruleeditor_delete_clicked(): No data appended to the statement.\n");
      return;
    }
    
    statement_remove(gui.ruleeditor.curruleset, sm);
    
    sellist = sellist->next;
  }
  
  /* Update the GUI. */
  gui_statementlist_update(store, gui.ruleeditor.curruleset);
}


/*
 * Move all currently selected items in the statement list up by one.
 */
void on_button_ruleeditor_up_clicked(void)
{
  GtkTreeView   *tree         = GTK_TREE_VIEW(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, "treeview_ruleeditor_rules"));
  GtkListStore  *store        = GTK_LIST_STORE(gtk_tree_view_get_model(tree));
  GList         *smlist       = NULL;
  
  gui_statementlist_selection_up(tree);
  smlist = gui_statementlist_order_get(store);
  
  g_list_free(gui.ruleeditor.curruleset->statements);
  gui.ruleeditor.curruleset->statements = smlist;
}


/*
 * Move all currently selected items in the statement list down by one.
 */
void on_button_ruleeditor_down_clicked(void)
{
  GtkTreeView   *tree         = GTK_TREE_VIEW(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, "treeview_ruleeditor_rules"));
  GtkListStore  *store        = GTK_LIST_STORE(gtk_tree_view_get_model(tree));
  GList         *smlist       = NULL;
  
  gui_statementlist_selection_down(tree);
  smlist = gui_statementlist_order_get(store);
  
  g_list_free(gui.ruleeditor.curruleset->statements);
  gui.ruleeditor.curruleset->statements = smlist;
}


/*
 * We use ONE function for almost all GUI callbacks (to be more specific:
 * only for those callbacks that indicate the change of a rule).
 * That's an easy way to always have the updated values in a nice struct.
 */
void on_ruleeditor_rule_widget_changed(GtkWidget *widget)
{
  const gchar *widgetname = gtk_widget_get_name(GTK_WIDGET(widget));
  gint        i = -1;
  
  g_assert(widgetname != NULL);
  
  while (options[++i].widgetname) {
    if (strcmp(widgetname, options[i].widgetname) != 0)
      continue;
    
    switch (options[i].type) {
    case TOGGLE:
      *(gint *)options[i].value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
      return;
      break;
    
    case ENTRY:
      *(const gchar **)options[i].value = (const gchar *)gtk_entry_get_text(GTK_ENTRY(widget));
      return;
      break;
    
    case OPTIONMENU:
      *(gint *)options[i].value = gtk_option_menu_get_history(GTK_OPTION_MENU(widget));
      
      /* Set the visibilty of some widget depending on the conditon option menu. */
      if (strcmp(widgetname, "optionmenu_ruleeditor_condition") == 0) {
        switch (*(gint *)options[i].value) {
        case 0:
          gtk_widget_hide(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, "entry_ruleeditor_condition_text"));
          gtk_widget_hide(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, "label_ruleeditor_condition_on"));
          gtk_widget_hide(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, "optionmenu_ruleeditor_condition_field"));
          break;
        
        default:
          gtk_widget_show(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, "entry_ruleeditor_condition_text"));
          gtk_widget_show(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, "label_ruleeditor_condition_on"));
          gtk_widget_show(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, "optionmenu_ruleeditor_condition_field"));
          break;
        }
      }
      return;
      break;
    
    default:
      g_assert_not_reached();
      break;
    }
  }
  
  fprintf(stderr, "Fatal Error: on_ruleeditor_rule_widget_changed():\n Widget not found: %s\n", widgetname);
}
/***************************************************************************************
 * END CALLBACKS
 ***************************************************************************************/


/*
 * Do the initialisation of this dialog whenever it is being reopened.
 */
void gui_ruleeditor_init(Ruleset *ruleset, Ruleset *oldruleset)
{
  GtkTreeView *tree = NULL;
  
  g_assert(ruleset != NULL);
  
  if (!gui.ruleeditor.ruleeditor)
    gui_ruleeditor_build();
  
  gui.ruleeditor.curruleset = ruleset;
  gui.ruleeditor.editingruleset = oldruleset;
  
  gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, "entry_ruleeditor_rulesetname")), gui.ruleeditor.curruleset->name);
  
  tree = GTK_TREE_VIEW(glade_xml_get_widget(gui.ruleeditor.ruleeditor_xml, "treeview_ruleeditor_rules"));
  g_assert(tree != NULL);
  
  /* Update the statement list (GUI). */
  gui_statementlist_update(GTK_LIST_STORE(gtk_tree_view_get_model(tree)), ruleset);
}


void gui_ruleeditor_destroy(void)
{
  gtk_widget_destroy(gui.ruleeditor.ruleeditor);
  gui.ruleeditor.ruleeditor = NULL;
  gui.ruleeditor.ruleeditor_xml = NULL;
}


/*
 * Create a new statement and add it to the current ruleset.
 */
void gui_ruleeditor_rule_add(Ruleset *ruleset)
{
  typedef struct {
    gboolean  enabled;
    int       type;
    short int data1type;
    void      *data1;
    short int data2type;
    void      *data2;
  } Rulemap;
  Rulemap rulelist[] = {
    /* Flags. */
    { gui.ruleeditor.strip_multiple_spaces,   RULE_STRIP_MULTIPLE_SPACES,   INT16, NULL, INT16, NULL },
    { gui.ruleeditor.remove_leading_spaces,   RULE_REMOVE_LEADING_SPACES,   INT16, NULL, INT16, NULL },
    { gui.ruleeditor.remove_trailing_spaces,  RULE_REMOVE_TRAILING_SPACES,  INT16, NULL, INT16, NULL },
    { gui.ruleeditor.remove_before_number,    RULE_REMOVE_BEFORE_NUMBER,    INT16, NULL, INT16, NULL },
    
    /* Capitol Letters. */
    { gui.ruleeditor.uppercase,               RULE_UPPERCASE,               INT16, NULL, INT16, NULL },
    { gui.ruleeditor.uppercase_word,          RULE_UPPERCASE_WORD,          INT16, NULL, INT16, NULL },
    { gui.ruleeditor.uppercase_first_word,    RULE_UPPERCASE_FIRST_WORD,    INT16, NULL, INT16, NULL },
    { gui.ruleeditor.lowercase,               RULE_LOWERCASE,               INT16, NULL, INT16, NULL },
    
    /* Static. */
    { gui.ruleeditor.set,             RULE_SET,             STR,    (void *)gui.ruleeditor.set_to,              INT16,  NULL },
    
    /* Modifiers. */
    { gui.ruleeditor.replace,         RULE_REPLACE,         STR,    (void *)gui.ruleeditor.replace_from,        STR,    (void *)gui.ruleeditor.replace_to },
    { gui.ruleeditor.replace_ci,      RULE_REPLACE_CI,      STR,    (void *)gui.ruleeditor.replace_ci_from,     STR,    (void *)gui.ruleeditor.replace_ci_to },
    
    /* Buffer Operations. */
    { gui.ruleeditor.buffer_grab,     RULE_BUFFER_GRAB,     STR,    (void *)gui.ruleeditor.buffer_grab_format,  INT16,  NULL },
    { gui.ruleeditor.buffer_copy_to,  RULE_BUFFER_COPY_TO,  INT16,  NULL,                                       INT16,  NULL },
    
    { FALSE, 0, INT16, NULL, INT16, NULL }
  };
  unsigned  short int rule_dest = 0;
  gboolean  rule_found = FALSE;
  gint      i = -1;
  
  g_assert(gui.ruleeditor.curruleset != NULL);
  
  /* Check if at least one rule has been selected.  Also, parse the rules for validity. */
  while (rulelist[++i].type || rulelist[i].data1 || rulelist[i].data2) {
    if (!rulelist[i].enabled)
      continue;
    if (rulelist[i].type == RULE_BUFFER_GRAB && !shellpattern_is_valid(rulelist[i].data1)) {
      gui_popup(gui.mainwindow, _("The pattern that you entered for the rule named \"Grab a string into the buffer using format\" is not valid.\n\n"
        "Please enter a pattern using a format like \"Artist - (*).mp3\" to grab the song name, for example."),
        GTK_MESSAGE_ERROR);
      return;
    }
    rule_found = TRUE;
  }
  if (!rule_found) {
    gui_popup(gui.mainwindow, _("No rule has been selected.\n\n"
      "Please select at least one rule and try again."), GTK_MESSAGE_ERROR);
    return;
  }
  
  /* Check if a rule destination has been selected. */
  rule_dest = ruleeditor_destination_get();
  if (!rule_dest) {
    gui_popup(gui.mainwindow, _("No rule destination selected.\n\n"
      "Please select where this rule should be applied to."), GTK_MESSAGE_ERROR);
    return;
  }
  
  /* Check if the condition pattern is valid. */
  if (gui.ruleeditor.cond_type + 1 == COND_SHELLPATTERN_MATCH && !shellpattern_is_valid(gui.ruleeditor.cond_text)) {
    gui_popup(gui.mainwindow, _("The pattern that you entered for the condition named \"Shell Pattern match\" is not valid.\n\n"
      "Please enter a pattern using a format like \"Artist - (*).mp3\" to match the song name, for example."),
      GTK_MESSAGE_ERROR);
    return;
  }
  
  /* Check if the condition regexp is valid. */
  if (gui.ruleeditor.cond_type + 1 == COND_REGEXP_MATCH && !regexp_is_valid(gui.ruleeditor.cond_text)) {
    gui_popup(gui.mainwindow, _("The pattern that you entered for the condition named \"RegExp match\" is not valid.\n\n"
      "Please enter a pattern using a format like \"^[A-Za-z]\" to match a field with at least one word, for example."),
      GTK_MESSAGE_ERROR);
    return;
  }
  
  /* Create a ruleset according to which rules are enabled in the GUI. */
  i = -1;
  while (rulelist[++i].type || rulelist[i].data1 || rulelist[i].data2) {
    if (!rulelist[i].enabled)
      continue;
    
    /* Append the current statement to the ruleset. */
    statement_append(gui.ruleeditor.curruleset, rule_dest,
      gui.ruleeditor.cond_type + 1, 1 << (gui.ruleeditor.cond_field), STR, (void*)gui.ruleeditor.cond_text,
      rulelist[i].type, rulelist[i].data1type, rulelist[i].data1, rulelist[i].data2type, rulelist[i].data2);
  }
}

#undef TYPES
