/* The Cantus project.
 * (c)2002, 2003 by Samuel Abels (spam debain org)
 * This project's homepage is: http://www.debain.org/cantus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <libintl.h>
#include "engine_init.h"
#include "engine_fileinfo.h"
#include "engine_jobs.h"
#include "engine_ruleset.h"
#include "gui_init.h"
#include "gui_queue.h"
#include "gui_genre.h"
#include "gui_tageditor.h"

#define _(String) gettext (String)
#define gettext_noop(String) (String)
#define N_(String) gettext_noop (String)

extern Cfg    cfg;
extern GUI    gui;

typedef struct {
  gint type;
  gchar *widgetname;
  gchar *value;
} Widgetmap;

enum TYPES {
  ENTRY,
  TEXTVIEW
};

/***************************************************************************************
 * STATICS
 ***************************************************************************************/
/*
 * Delete the content of a textview widget.
 */
static void del_textview_text(GtkTextView *tv)
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter start;
  GtkTextIter end;
  
  g_return_if_fail(tv != NULL);
  
  buffer = gtk_text_view_get_buffer(tv);
  
  g_return_if_fail(buffer != NULL);
  
  gtk_text_buffer_get_start_iter(buffer, &start);
  gtk_text_buffer_get_end_iter(buffer, &end);
  gtk_text_buffer_delete((GtkTextBuffer *)buffer, &start, &end);
}


static void textview_append_text(GtkTextView * tv, const char *text)
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter end;
  
  buffer = gtk_text_view_get_buffer(tv);
  g_return_if_fail(buffer != NULL);
  
  /* Append the text to the buffer. */
  gtk_text_buffer_get_end_iter(buffer, &end);
  gtk_text_buffer_insert(buffer, &end, text, -1);
}


static void gui_tageditor_update_1(Tag *tag)
{
  GtkWidget *widget = NULL;
  Widgetmap map[] = {
    { ENTRY,    "entry_tageditor_artist",     tag->artist },
    { ENTRY,    "entry_tageditor_song",       tag->title },
    { ENTRY,    "entry_tageditor_album",      tag->album },
    { ENTRY,    "entry_tageditor_track",      tag->track },
    { ENTRY,    "entry_tageditor_year",       tag->year },
    { ENTRY,    "entry_tageditor_genre",      tag->genre },
    { TEXTVIEW, "textview_tageditor_comment", tag->comment },
    { 0, NULL, NULL }
  };
  gint i = 0;
  
  while (map[i].widgetname) {
    widget = GTK_WIDGET(glade_xml_get_widget(gui.mainwindow_xml, map[i].widgetname));
    
    switch (map[i].type) {
    case ENTRY:
      gtk_entry_set_text(GTK_ENTRY(widget), map[i].value);
      break;
    
    case TEXTVIEW:
      del_textview_text(GTK_TEXT_VIEW(widget));
      textview_append_text(GTK_TEXT_VIEW(widget), map[i].value);
      break;
    }
    
    i++;
  }
}


/*
 * Return the content of the comment widget.
 */
static gchar *gui_tageditor_comment_get(void)
{
  GtkTextView *tv = GTK_TEXT_VIEW(glade_xml_get_widget(gui.mainwindow_xml, "textview_tageditor_comment"));
  GtkTextBuffer *buffer = NULL;
  GtkTextIter start;
  GtkTextIter end;
  
  g_return_val_if_fail(tv != NULL, NULL);
  
  buffer = gtk_text_view_get_buffer(tv);
  
  g_return_val_if_fail(buffer != NULL, NULL);
  
  gtk_text_buffer_get_start_iter(buffer, &start);
  gtk_text_buffer_get_end_iter(buffer, &end);
  
  return(gtk_text_buffer_get_text((GtkTextBuffer *)buffer, &start, &end, FALSE));
}


static PreviewDestroyFunc on_preview_destroy(void)
{
  gui.preview = NULL;
}
/***************************************************************************************
 * END STATICS
 ***************************************************************************************/


/***************************************************************************************
 * CALLBACKS
 ***************************************************************************************/
void on_entry_tageditor_artist_changed(GtkEntry *entry, gpointer trash)
{
  gui.tageditor.artist = gtk_entry_get_text(entry);
}


void on_entry_tageditor_song_changed(GtkEntry *entry, gpointer trash)
{
  gui.tageditor.song = gtk_entry_get_text(entry);
}


void on_entry_tageditor_album_changed(GtkEntry *entry, gpointer trash)
{
  gui.tageditor.album = gtk_entry_get_text(entry);
}


void on_entry_tageditor_track_changed(GtkEntry *entry, gpointer trash)
{
  gui.tageditor.track = gtk_entry_get_text(entry);
}


void on_entry_tageditor_year_changed(GtkEntry *entry, gpointer trash)
{
  gui.tageditor.year = gtk_entry_get_text(entry);
}


void on_entry_tageditor_genre_changed(GtkEntry *entry, gpointer trash)
{
  gui.tageditor.genre = gtk_entry_get_text(entry);
}


void on_button_tageditor_genre_clicked(GtkButton *button, gpointer trash)
{
  if (!gui.genre)
    gui_genre_init();
  gtk_widget_show(gui.genre);
  
  gui_genre_selection_init(gui.tageditor.genre);
}


void on_button_tageditor_save_clicked(GtkButton *button, gpointer trash)
{
  JobArg arg;
  
  arg.ruleset = ruleset_new(NULL);
  arg.preview = cfg.preview_tageditor;
  
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui.mainwindow_xml, "check_tageditor_artist"))))
    statement_append(arg.ruleset, F_ARTIST, COND_ALWAYS, 0, INT16, NULL, RULE_SET, STR, (void *)gui.tageditor.artist,  INT16, (void *)NULL);
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui.mainwindow_xml, "check_tageditor_song"))))
    statement_append(arg.ruleset, F_SONG,   COND_ALWAYS, 0, INT16, NULL, RULE_SET, STR, (void *)gui.tageditor.song,    INT16, (void *)NULL);
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui.mainwindow_xml, "check_tageditor_album"))))
    statement_append(arg.ruleset, F_ALBUM,  COND_ALWAYS, 0, INT16, NULL, RULE_SET, STR, (void *)gui.tageditor.album,   INT16, (void *)NULL);
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui.mainwindow_xml, "check_tageditor_track"))))
    statement_append(arg.ruleset, F_TRACK,  COND_ALWAYS, 0, INT16, NULL, RULE_SET, STR, (void *)gui.tageditor.track,   INT16, (void *)NULL);
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui.mainwindow_xml, "check_tageditor_year"))))
    statement_append(arg.ruleset, F_YEAR,   COND_ALWAYS, 0, INT16, NULL, RULE_SET, STR, (void *)gui.tageditor.year,    INT16, (void *)NULL);
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui.mainwindow_xml, "check_tageditor_genre"))))
    statement_append(arg.ruleset, F_GENRE,  COND_ALWAYS, 0, INT16, NULL, RULE_SET, STR, (void *)gui.tageditor.genre,   INT16, (void *)NULL);
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui.mainwindow_xml, "check_tageditor_comment"))))
    statement_append(arg.ruleset, F_COMMENT, COND_ALWAYS, 0,INT16, NULL, RULE_SET, STR, (void *)gui_tageditor_comment_get(), INT16, (void *)NULL);
  
  job_preview_clear();
  gui_queue_selection_for_each(GTK_TREE_VIEW(glade_xml_get_widget(gui.mainwindow_xml, "treeview_queue")), (void*)job_append, &arg);
  engine_wakeup();
  
  if (cfg.preview_tageditor && !gui.preview)
    gui.preview = gui_preview_init((PreviewDestroyFunc)on_preview_destroy);
}


void on_button_tageditor_delete_clicked(GtkButton *button, gpointer trash)
{
  JobArg arg;
  
  arg.ruleset = ruleset_new(NULL);
  arg.preview = FALSE;
  
  statement_append(arg.ruleset, F_ARTIST,   COND_ALWAYS, 0, INT16, NULL, RULE_SET, STR, "", INT16, (void *)NULL);
  statement_append(arg.ruleset, F_SONG,     COND_ALWAYS, 0, INT16, NULL, RULE_SET, STR, "", INT16, (void *)NULL);
  statement_append(arg.ruleset, F_ALBUM,    COND_ALWAYS, 0, INT16, NULL, RULE_SET, STR, "", INT16, (void *)NULL);
  statement_append(arg.ruleset, F_TRACK,    COND_ALWAYS, 0, INT16, NULL, RULE_SET, STR, "", INT16, (void *)NULL);
  statement_append(arg.ruleset, F_YEAR,     COND_ALWAYS, 0, INT16, NULL, RULE_SET, STR, "", INT16, (void *)NULL);
  statement_append(arg.ruleset, F_GENRE,    COND_ALWAYS, 0, INT16, NULL, RULE_SET, STR, "", INT16, (void *)NULL);
  statement_append(arg.ruleset, F_COMMENT,  COND_ALWAYS, 0, INT16, NULL, RULE_SET, STR, "", INT16, (void *)NULL);
  
  gui_queue_selection_for_each(GTK_TREE_VIEW(glade_xml_get_widget(gui.mainwindow_xml, "treeview_queue")), (void*)job_append, &arg);
  engine_wakeup();
}
/***************************************************************************************
 * END CALLBACKS
 ***************************************************************************************/


void gui_tageditor_init(void)
{
  memset(&gui.tageditor, 0, sizeof(Tageditor));
}


void gui_tageditor_clear(void)
{
  Tag tag;
  
  memset(&tag, 0, sizeof(Tag));
  gui_tageditor_update_1(&tag);
  
  return;
}


void gui_tageditor_update(void)
{
  GtkTreeView *queue = GTK_TREE_VIEW(glade_xml_get_widget(gui.mainwindow_xml, "treeview_queue"));
  GtkLabel *label = GTK_LABEL(glade_xml_get_widget(gui.mainwindow_xml, "label_fileinfo"));
  GtkTreeSelection *selection = NULL;
  GtkTreeModel *model = NULL;
  GList *sellist = NULL;
  GList *item = NULL;
  const FileInfo *info = NULL;
  Tag * srctag = NULL;
  Tag tag;
  
  if (!(selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(queue))))
    return;
  
  if (!(sellist = gtk_tree_selection_get_selected_rows(selection, &model)))
    return;
  g_assert(model != NULL);
  
  /* Get the first tag info. */
  item = sellist;
  if (!(info = gui_queue_row_info_get(GTK_TREE_VIEW(queue), (GtkTreePath*)item->data))) {
    fprintf(stderr, "Error: tageditor_update(): Fileinfo pointer is NULL.");
    return;
  }
  /* Update the summary. */
  if (!item->next)
    gtk_label_set_text(label, info->summary);
  else {
    gchar summary[2048];
    snprintf(summary, 2047, _("%i files selected.\n"), g_list_length(sellist));
    gtk_label_set_text(label, summary);
  }
  
  if (!(srctag = info->tag[cfg.defaulttag_v2]))
    srctag = info->tag[cfg.defaulttag_v2 ^ 1];
  
  if (!srctag)
    return;
  
  memcpy(&tag, srctag, sizeof(Tag));
  
  /* Now parse through all tags and clear the GUI fields if something doesn't equal the first tag. */
  item = item->next;
  while (item) {
    /* Get the fileinfo data pointer of the given gtk path. */
    if (!(info = gui_queue_row_info_get(GTK_TREE_VIEW(queue), (GtkTreePath*)item->data))) {
      item = item->next;
      continue;
    }
    
    /* Merge the tag info. */
    if (!(srctag = info->tag[cfg.defaulttag_v2]))
      srctag = info->tag[cfg.defaulttag_v2 ^ 1];
    
    if (!srctag)
      return;
    
    if (strcmp(tag.artist, srctag->artist) != 0)
      *tag.artist = '\0';
    if (strcmp(tag.title, srctag->title) != 0)
      *tag.title = '\0';
    if (strcmp(tag.album, srctag->album) != 0)
      *tag.album = '\0';
    if (strcmp(tag.track, srctag->track) != 0)
      *tag.track = '\0';
    if (strcmp(tag.year, srctag->year) != 0)
      *tag.year = '\0';
    if (strcmp(tag.genre, srctag->genre) != 0)
      *tag.genre = '\0';
    if (strcmp(tag.comment, srctag->comment) != 0)
      *tag.comment = '\0';
    
    item = item->next;
  }
  
  g_list_foreach(sellist, (GFunc)gtk_tree_path_free, NULL);
  g_list_free(sellist);
  
  gui_tageditor_update_1(&tag);
}


void gui_tageditor_genre_set(const gchar *text)
{
  GtkEntry *entry = GTK_ENTRY(glade_xml_get_widget(gui.mainwindow_xml, "entry_tageditor_genre"));
  gtk_entry_set_text(entry, text);
}
