/* The Cantus project.
 * (c)2002, 2003 by Samuel Abels (spam debain org)
 * This project's homepage is: http://www.debain.org/cantus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/stat.h>
#include "lib_recurse.h"
#include "lib_charset.h"
#include "lib_shellpattern.h"


/***************************************************************************************
 * STATICS
 ***************************************************************************************/
/*
 * Returns TRUE if *path is a directory, FALSE if not.
 */
static short int is_dir(char *path)
{
  struct stat pstat;
  
  if (stat(path, &pstat) < 0)
    return(FALSE);
  
  return(S_ISDIR(pstat.st_mode) != 0);
}
/***************************************************************************************
 * END STATICS
 ***************************************************************************************/


void recursive_for_each_file(const char *directory, const char *pattern[], void (* func)(const char *filename, void *data), void *data)
{
  DIR           *stream             = NULL;
  struct dirent *filestruct         = NULL;
  char          fullfilename[4096];
  
  /* Open the directory. */
  if (!(stream = opendir(directory))) {
    printf("Error: recursive_for_each_file(): Failed opening path. (%s)\n", directory);
    return;
  }
  
  /* Parse through the directory. */
  while ((filestruct = readdir(stream))) {
    /* Don't follow the relative directory links. */
    if (strcmp(filestruct->d_name, "..") == 0 || strcmp(filestruct->d_name, ".") == 0)
      continue;
    
    /* Create a fullpath. */
    snprintf(fullfilename, 4095, "%s%s", directory, filestruct->d_name);
    
    /* If this is a directory, call yourself (recursive). */
    if (is_dir(fullfilename)) {
      strcpy(fullfilename + strlen(fullfilename), "/");
      recursive_for_each_file(fullfilename, pattern, func, data);
      continue;
    }
    
    /* Does the file match the pattern? */
    if (pattern && !shellpattern_match_any(filestruct->d_name, pattern, FALSE))
      continue;
    
    /* Call the user's function. */
    func(fullfilename, data);
  }
  
  closedir(stream);
}
