/* The Cantus project.
 * (c)2002, 2003 by Samuel Abels (spam debain org)
 * This project's homepage is: http://www.debain.org/cantus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "lib_strings.h"


/***************************************************************************************
 * STATICS
 ***************************************************************************************/
/***************************************************************************************
 * END STATICS
 ***************************************************************************************/


char *strstr_ci(const char *str, const char *needle)
{
  char *pstr = (char *)str;
  while (*pstr) {
    if (strncasecmp(pstr, needle, strlen(needle)) == 0)
      return(pstr);
    pstr++;
  }
  return(NULL);
}


/*
 * Replace "from" by "to".
 */
void str_replace(char *str, const char *from, const char *to, int maxlen)
{
  char  *pstr   = str;
  int   fromlen = strlen(from);
  int   tolen   = strlen(to);
  
  while (*pstr != '\0' && pstr - str < maxlen) {
    if (strncmp(pstr, from, fromlen) != 0) {
      *pstr++;
      continue;
    }
    memmove(pstr + tolen, pstr + fromlen, maxlen - ((pstr + tolen) - str) - 1);
    memcpy(pstr, to, tolen);
    pstr += tolen;
  }
}


/*
 * Replace "from" by "to". (case insensitive)
 */
void str_replace_ci(char *str, const char *from, const char *to, int maxlen)
{
  char  *pstr   = str;
  int   fromlen = strlen(from);
  int   tolen   = strlen(to);
  
  while (*pstr != '\0' && pstr - str < maxlen) {
    if (strncasecmp(pstr, from, fromlen) != 0) {
      *pstr++;
      continue;
    }
    memmove(pstr + tolen, pstr + fromlen, maxlen - ((pstr + tolen) - str) - 1);
    memcpy(pstr, to, tolen);
    pstr += tolen;
  }
}


/*
 * Uppercase a string.
 */
void strtoupper(char *str)
{
  if (!str)
    return;
  while (*str != '\0') {
    if (isalpha(*str))
      *str = toupper(*str);
    str++;
  }
}


/*
 * Lowercase a string.
 */
void strtolower(char *str)
{
  if (!str)
    return;
  while (*str != '\0') {
    if (isalpha(*str))
      *str = tolower(*str);
    str++;
  }
}


/*
 * Change the characters case.
 * mode=0:first word uppercase;
 * mode=1:every word uppercase;
 */
void strtoupper_partly(char *str, int mode)
{
	if(!str)
    return;
  
  /* Uppercase the first word. */
  while (*str != '\0') {
    if (!isalpha(*str)) {
      str++;
      continue;
    }
    *str = toupper(*str);
    str++;
    break;
  }
  
  if (mode == 0 || *str == '\0')
    return;
  
  /* Uppercase all other words. */
  while (*str != '\0') {
    if (isalpha(*(str - 1)) || !isalpha(*str)) {
      str++;
      continue;
    }
    *str = toupper(*str);
    str++;
  }
}


/*
 * This function will strip leading/trailing/both spaces in "source".
 * Spaces are:
 * Single space, tab, vertical tab, form feed, carriage return, or newline.
 * loc: 0-start 1-end 2-both
 */ 
void string_wipe_spaces(char *str, int loc) 
{
  char *pstr = str;
  
  if (!str)
    return;
  
  /* Remove leading spaces. */
  if (loc == 0 || loc == 2) {
    while (*pstr == ' ')
      pstr++;
    memmove(str, pstr, strlen(str) - (pstr - str) + 1);
  }
  /* Remove trailing spaces. */
  if (loc == 1 || loc == 2) {
    for (pstr = str + strlen(str) - 1; pstr != str; pstr--) {
      if (!isspace(*pstr))
        break;
      *pstr = '\0';
    }
  }
  
  return;
}


/*
 * Split a string that is delimited by any of the chars in "delimiters" (max 1000) into an array.
 */
char **explode(const char *string, const char *delimiters)
{
  char **strlist = calloc(1000, sizeof(char));
  char *stringcp = string ? strdup(string) : NULL;
  int i = 0;
  
  if (!string || *string == '\0')
    return(strlist);
  
  /* Init strtok. */
  strlist[0] = strdup(strtok(stringcp, delimiters));
  
  /* Split into pieces. */
  while ((strlist[++i] = strtok(NULL, delimiters)))
    strlist[i] = strdup(strlist[i]);
  
  free(stringcp);
  
  return(strlist);
}


/*
 * Free all elements of an array.
 */
void free_array(char **array)
{
  int i = -1;
  
  while (array[++i])
    free(array[i]);
  
  free(array);
}


/*
 * Find the given string in any element of an array. Returns a pointer to the element.
 */
const char *array_element_case_insensitive(const char **array, const char *needle)
{
  int i = -1;
  
  while (array[++i])
    if (strcasecmp(array[i], needle) == 0)
      return(array[i]);
  
  return(NULL);
}


/*
 * Count the number of elements in an array.
 */
int array_num_elements(const char **array)
{
  int i = -1;
  
  while (array[++i]);
  
  return(i);
}
